// TreeIterator utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

package com.myarch.treeiterator;

import java.util.Iterator;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.DOMException;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Document;

/**
 * This adapter maps <code>org.w3c.dom.Node</code> interface to
 * the generic methods of the <code>TreeIteratorAdapter</code>.
 * <code>java.util.Iterator</code> interface implementation is provided by the
 * <code>ChildrenAdapter</code> nested class since it is not directly supported
 * by <code>org.w3c.dom.Node</code>.
 */
public class DOMAdapter implements TreeNodeAdapter {

  /**
   * Returns <code>java.util.Iterator</code> to iterate over children of the node.
   * @param node node
   * @return <code>java.util.Iterator</code> for the list of children.
   */
  public Iterator getChildren( Object node ) {
    return new ChildrenAdapter( ((Node)node).getFirstChild() );
  }

  /**
   * Returns true if the node has children.
   * @param node node
   * @return <code>true</code> if the node has children
   */
  public boolean hasChildren( Object node ) {
    return ((Node)node).hasChildNodes();
  }

  /**
   * Implements  <code>java.util.Iterator</code> methods for the
   * node list.
   */
  static public class ChildrenAdapter implements Iterator {
    private Node node;

    ChildrenAdapter( Node node) {
      // empty node is the head of the list
      this.node = new HeadNode( node );
    }

    /**
     * Returns the next element in the interation.
     * @return the next element in the iteration or <code>null</code> if there
     * are no more elements.
     */
    public Object next() {
      node = node.getNextSibling();
      return node;
    }

   /**
    * Returns true if the iteration has more elements.
    * @return true if the iterator has more elements.
    */
    public boolean hasNext() {
      return (node.getNextSibling() != null);
    }

   /**
    * This method is defined to comply with <code>Iterator</code> but it is not supported.
    * @throws UnsupportedOperationException whenever it is called
    */
    public void remove() throws UnsupportedOperationException {
     throw new UnsupportedOperationException("Remove is not supported by DOMAdapter");
    }


    /**
     * Dummy implementation of the <code>Node</code>.
     * This is needed in order to advance to the first element after the <code>next()</code> call.
     * It returns its node after the <code>nextSibling</code> call
     */
    private class HeadNode  implements Node {
      private Node nextNode;

      /**
       * Creates the node and initiates with the current node
       */
      private HeadNode( Node node){
        nextNode = node;
      }

      /**
       * Returns current node as the next.
       * This is the only method that's used out of <code>Node</code> interface.
       * @return current node
       */
      public Node getNextSibling() {
        return nextNode;
      }

      // all following methods are not used
      public String getNodeName(){ return null;}
      public String getNodeValue() throws DOMException { return null;}
      public void setNodeValue(String nodeValue) throws DOMException {}
      public short getNodeType() {return 0;}
      public Node getParentNode() {return null;}
      public NodeList getChildNodes() {return null;}
      public Node getFirstChild()  {return null;}
      public Node getLastChild()  {return null;}
      public Node getPreviousSibling()  {return null;}
      public NamedNodeMap getAttributes(){return null;}
      public Document getOwnerDocument(){return null;}
      public Node insertBefore(Node newChild, Node refChild)
        throws DOMException {return null;}
      public Node replaceChild(Node newChild, Node oldChild)
        throws DOMException {return null;}
      public Node removeChild(Node oldChild)
        throws DOMException {return null;}
      public Node appendChild(Node newChild)
        throws DOMException {return null;}
      public boolean hasChildNodes() {return false;}
      public Node cloneNode(boolean deep) {return null;}

      // XML2 methods
      public String getNamespaceURI(){ return null;}
      public String getLocalName(){ return null; }
      public String getPrefix(){ return null;}
      public boolean hasAttributes() {return false; }
      public void normalize() {;}
      public void setPrefix( String prefix)  throws DOMException { ;}
      public boolean supports( String feature, String version) {return false; }
      public boolean isSupported( String feature, String version) {return false; }

    } // end of HeadNode

  } // end of IteratorCollection

}