
// TreeIterator test harness
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.


import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.FileReader;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.apache.xerces.parsers.DOMParser;

import com.myarch.treeiterator.TreeIterator;
import com.myarch.treeiterator.DOMAdapter;


import junit.framework.TestCase;


/**
 * TreeIterator test harness.
 * Junit (www.junit.org) framework is used to run the tests.
 *
 * This code uses Xerces (www.apache.org) parser. Change import and invocation if other parser is used.
 *
 * @author Alexander Ananiev
 */
public class Tester extends TestCase {

  public final static String TEST_XML_FILE = "personnel.xml";

  private DOMAdapter domAdapter = new DOMAdapter();
  static Document xdoc;

  public Tester( String name ) {
    super( name );
  }

  /**
   * Opens file with test data, creates DOM tree.
   */
  protected void setUp() throws Exception {
    DOMParser parser = new DOMParser();
    parser.parse( new InputSource( new FileReader( TEST_XML_FILE )));
    xdoc = parser.getDocument();

  }

  /**
   *  Tests full tree traversal.
   */
  public void testFull() {

    TreeIterator tIterator = new TreeIterator ( xdoc , domAdapter );
    int cnt=0;
    int maxDepth =-1;
    Node node = null;

    // test full traversal
    while ((node=(Node)tIterator.next())!=null ){
      //printNode( node, tIterator );
      ++cnt;
      if (tIterator.getDepth()> maxDepth )
         maxDepth =tIterator.getDepth();
    }
    System.out.println("Count:"+cnt);
    assert( "full traversal", (cnt==54 && maxDepth==5));
  }

  /**
   *  Tests selective traversal using <code>skipChildren</code>.
   *  Only the specific branch should be traversed.
   */
  public void testSkip() {

    // branch traversal
    int level =2;
    String attrName="id";
    String attrValue="N.URAMOTO";
    // test skip
    int cnt=0;
    int maxDepth =-1;
    TreeIterator tIterator = new TreeIterator ( xdoc , domAdapter );
    Node node = null;
    while ((node=(Node)tIterator.next())!=null ){
      ++cnt;
      if (tIterator.getDepth()> maxDepth )
         maxDepth =tIterator.getDepth();
      if  ( tIterator.getDepth() == 2 && ! (
        node instanceof Element && ((Element)node).getAttribute( attrName).
        equalsIgnoreCase( attrValue ))) {
        // allow parsing all children of the node -- applies only to the current node
        tIterator.skipChildren();
      }
    }

    System.out.println("Count:"+cnt);
    assert( "skip traversal", (cnt==23 && maxDepth==5));
  }

  /**
   *  Tests selective traversal using <code>setLevelDepth</code>
   *  (<code>moveToChildren</code>).
   *  Only the specific branch should be traversed.
   */
  public void testChildren() {

    // branch traversal
    int level =2;
    String attrName="id";
    String attrValue="N.URAMOTO";
    // test skip
    int cnt=0;
    int maxDepth =-1;
    TreeIterator tIterator = new TreeIterator ( xdoc , domAdapter, level );
    Node node = null;
    while ((node=(Node)tIterator.next())!=null ){
      ++cnt;
      if (tIterator.getDepth()> maxDepth )
         maxDepth =tIterator.getDepth();
      if  (node instanceof Element && ((Element)node).getAttribute( attrName).
        equalsIgnoreCase( attrValue )) {
        // allow parsing all children of the node -- applies only to the current node
        tIterator.moveToChildren();
       }
    }

    System.out.println("Count:"+cnt);
    assert( "moveToChildren traversal", (cnt==23 && maxDepth==5));
  }

  /**
   * Helper method to prrint XML node. Iterator is used to obtain level and index.
   */
  private void printNode( Node node, TreeIterator tIterator ) {
    System.out.print( tIterator.getDepth()+":" +tIterator.getIndex()+":");
    System.out.print( node.getNodeName());
    // do not print empty values
    if (node instanceof Text && !node.getNodeValue().trim().equals("")) {
      System.out.print( "="+node.getNodeValue() );
    }
    System.out.println();
  }


  public static void main(String[] args) {
    try {

      junit.textui.TestRunner.run( Tester.class );

    }
    catch( Exception ex ) {
      System.err.println("TreeTH: Exception:");
      ex.printStackTrace();

    }

  }
}
